package com.belkast.soap;

import java.io.*;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;
import java.nio.charset.CodingErrorAction;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class fileProcessor {

    public static String readFirstLine(String thisFile) throws IOException {
        File file = new File(thisFile);
        Boolean isValid = validFile(thisFile);

        if ( !isValid )
        {
            return null;
        }

        if (file.exists()) {
            FileReader fr = new FileReader(file);
            LineNumberReader ln = new LineNumberReader(fr);
            while (ln.getLineNumber() == 0) {
                String s = ln.readLine();
                return s;
            }
        }

        return null;

    }

    public static Boolean getFile(String myFile) {
        if (myFile == null) {
            return false;
        }

        File thisFile = new File(myFile);
        if (thisFile.exists()) {
            return true;
        }

        return false;
    }

    public static String replaceInFile(String myFile, Map hashMap) {
        File thisFile = new File(myFile);
        if (!thisFile.exists()) {
            return null;
        }
        StringBuilder sb = new StringBuilder();
        try {
            FileReader fileReader = new FileReader(myFile);
            BufferedReader reader = new BufferedReader(fileReader);
            String line;
            while ((line = reader.readLine()) != null)
            {
                line = line.replaceAll("(?m)^\\s+|\\s+$", "");
                sb.append(line);
            }
            reader.close();
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }

        String returnString = sb.toString();

        Set set = hashMap.entrySet();
        Iterator i = set.iterator();
        while (i.hasNext()) {
            Map.Entry me = (Map.Entry) i.next();
            returnString = returnString.replaceAll(me.getKey().toString(), me.getValue().toString());
        }

        return returnString;
    }

    public static boolean fileHasText(String thisFile, String word) throws FileNotFoundException
    {
        boolean flag = false;
        Scanner sc2 = new Scanner(new FileInputStream(thisFile));
        while(sc2.hasNextLine()) {
            String line = sc2.nextLine();
            if( line.indexOf(word) != -1 )
            {
                flag = true;
            }
        }
        return flag;
    }

    public static boolean stringHasText(String line, String regex) throws FileNotFoundException
    {
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(line);

        int matches = 0;
        while ( matcher.find() )
        {
            matches++;
        }
        return (matches > 0);
    }

    public static boolean validFile(String thisFile)
    {
        Path path = Paths.get(thisFile);
        boolean result = true;
        long total = 0;
        InputStreamReader isr = null;
        char[] buf = new char[65536];
        int chars;

        final CharsetDecoder decoder;
        String charsetName = "UTF-8";
        Charset charset = Charset.forName(charsetName);
        decoder = charset.newDecoder();
        decoder.onMalformedInput(CodingErrorAction.REPORT);

        try {
            isr = new InputStreamReader(new FileInputStream(thisFile), decoder);
            while ((chars = isr.read(buf)) >= 0)
                total += chars;
        } catch (CharacterCodingException ex) {
            result = false;
        } catch (IOException ex) {
            result = false;
        } finally {
            if (isr != null)
                try {
                    isr.close();
                } catch (IOException ex) {
                    System.err.println("Failed to close file: " + ex.toString());
                }
        }
        return result;
    }

    public static long numberOfLines(String fileName)
    {
        Path path = Paths.get(fileName);
        long lines = 0;

        try
        {
            lines = Files.lines(path).count();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }

        return lines;
    }
}